package HTTP::OAI;

use HTTP::OAI::Harvester;

use vars qw( $VERSION );
$HTTP::OAI::VERSION = $HTTP::OAI::Harvester::VERSION;

1;

__END__

=head1 NAME

HTTP::OAI - API for the OAI-PMH

=head1 DESCRIPTION

This is a stub module, you probably want to look at L<HTTP::OAI::Harvester|HTTP::OAI::Harvester> or L<HTTP::OAI::Repository|HTTP::OAI::Repository>.

=head1 AUTHOR

Copyright 2004 Tim Brody <tdb01r@ecs.soton.ac.uk>

These modules are distributed under the same terms as Perl.
